@@ -11,7 +11,7 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 # License for the specific language governing permissions and limitations
 # under the License.
-
+import hashlib
 import operator
 import tempfile
 import uuid
@@ -37,10 +37,11 @@ class BaseTestImage(base.TestCase):
         self.imagefile = tempfile.NamedTemporaryFile(delete=False)
         self.imagefile.write(b'\0')
         self.imagefile.close()
-        self.fake_image_dict = fakes.make_fake_image(
-            image_id=self.image_id, image_name=self.image_name)
-        self.fake_search_return = {'images': [self.fake_image_dict]}
         self.output = uuid.uuid4().bytes
+        self.fake_image_dict = fakes.make_fake_image(
+            image_id=self.image_id, image_name=self.image_name,
+            checksum=hashlib.md5(self.output).hexdigest())
+        self.fake_search_return = {'images': [self.fake_image_dict]}
         self.container_name = self.getUniqueString('container')
 
 
@@ -83,7 +84,15 @@ class TestImage(BaseTestImage):
     def test_download_image_no_images_found(self):
         self.register_uris([
             dict(method='GET',
-                 uri='https://image.example.com/v2/images',
+                 uri='https://image.example.com/v2/images/{name}'.format(
+                     name=self.image_name),
+                 status_code=404),
+            dict(method='GET',
+                 uri='https://image.example.com/v2/images?name={name}'.format(
+                     name=self.image_name),
+                 json=dict(images=[])),
+            dict(method='GET',
+                 uri='https://image.example.com/v2/images?os_hidden=True',
                  json=dict(images=[]))])
         self.assertRaises(exc.OpenStackCloudResourceNotFound,
                           self.cloud.download_image, self.image_name,
@@ -93,13 +102,21 @@ class TestImage(BaseTestImage):
     def _register_image_mocks(self):
         self.register_uris([
             dict(method='GET',
-                 uri='https://image.example.com/v2/images',
+                 uri='https://image.example.com/v2/images/{name}'.format(
+                     name=self.image_name),
+                 status_code=404),
+            dict(method='GET',
+                 uri='https://image.example.com/v2/images?name={name}'.format(
+                     name=self.image_name),
                  json=self.fake_search_return),
             dict(method='GET',
                  uri='https://image.example.com/v2/images/{id}/file'.format(
                      id=self.image_id),
                  content=self.output,
-                 headers={'Content-Type': 'application/octet-stream'})
+                 headers={
+                     'Content-Type': 'application/octet-stream',
+                     'Content-MD5': self.fake_image_dict['checksum']
+                 })
         ])
 
     def test_download_image_with_fd(self):
@@ -147,7 +164,7 @@ class TestImage(BaseTestImage):
                      base_url_append='v2'),
                  json=self.fake_image_dict)
         ])
-        self.assertEqual(
+        self.assertDictEqual(
             self.cloud._normalize_image(self.fake_image_dict),
             self.cloud.get_image_by_id(self.image_id))
         self.assert_calls()
@@ -326,6 +343,12 @@ class TestImage(BaseTestImage):
                      'image', append=['images', self.image_id, 'file'],
                      base_url_append='v2'),
                  request_headers={'Content-Type': 'application/octet-stream'}),
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     'image', append=['images', self.fake_image_dict['id']],
+                     base_url_append='v2'
+                 ),
+                 json=self.fake_image_dict),
             dict(method='GET',
                  uri=self.get_mock_url(
                      'image', append=['images'], base_url_append='v2'),
@@ -411,7 +434,7 @@ class TestImage(BaseTestImage):
             dict(method='POST',
                  uri=self.get_mock_url(
                      'image', append=['tasks'], base_url_append='v2'),
-                 json=args,
+                 json={'id': task_id, 'status': 'processing'},
                  validate=dict(
                      json=dict(
                          type='import', input={
@@ -430,8 +453,9 @@ class TestImage(BaseTestImage):
                  json=args),
             dict(method='GET',
                  uri=self.get_mock_url(
-                     'image', append=['images'], base_url_append='v2'),
-                 json={'images': [image_no_checksums]}),
+                     'image', append=['images', self.image_id],
+                     base_url_append='v2'),
+                 json=image_no_checksums),
             dict(method='PATCH',
                  uri=self.get_mock_url(
                      'image', append=['images', self.image_id],
@@ -447,10 +471,11 @@ class TestImage(BaseTestImage):
                           u'path': u'/owner_specified.openstack.md5'},
                          {u'op': u'add', u'value': fakes.NO_SHA256,
                           u'path': u'/owner_specified.openstack.sha256'}],
-                         key=operator.itemgetter('value')),
+                         key=operator.itemgetter('path')),
                      headers={
                          'Content-Type':
-                             'application/openstack-images-v2.1-json-patch'})
+                             'application/openstack-images-v2.1-json-patch'}),
+                 json=self.fake_search_return
                  ),
             dict(method='HEAD',
                  uri='{endpoint}/{container}/{object}'.format(
@@ -471,14 +496,6 @@ class TestImage(BaseTestImage):
                  uri='{endpoint}/{container}/{object}'.format(
                      endpoint=endpoint, container=self.container_name,
                      object=self.image_name)),
-            dict(method='GET',
-                 uri=self.get_mock_url(
-                     'image', append=['images'], base_url_append='v2'),
-                 json=self.fake_search_return),
-            # TODO(mordred) The task workflow results in an extra call
-            # in the upper level wait. We should be able to make this
-            # go away once we refactor a wait_for_image out in the next
-            # patch.
             dict(method='GET',
                  uri=self.get_mock_url(
                      'image', append=['images'], base_url_append='v2'),
@@ -487,6 +504,7 @@ class TestImage(BaseTestImage):
 
         self.cloud.create_image(
             self.image_name, self.imagefile.name, wait=True, timeout=1,
+            disk_format='vhd', container_format='ovf',
             is_public=False, container=self.container_name)
 
         self.assert_calls()
@@ -633,7 +651,8 @@ class TestImage(BaseTestImage):
                     'owner_specified.openstack.sha256': fakes.NO_SHA256,
                     'owner_specified.openstack.object': 'images/{name}'.format(
                         name=self.image_name),
-                    'is_public': False}}
+                    'is_public': False},
+                'validate_checksum': True}
 
         ret = args.copy()
         ret['id'] = self.image_id
@@ -734,6 +753,52 @@ class TestImage(BaseTestImage):
 
         self.assert_calls()
 
+    def test_create_image_put_v2_wrong_checksum_delete(self):
+        self.cloud.image_api_use_tasks = False
+
+        args = {'name': self.image_name,
+                'container_format': 'bare', 'disk_format': 'qcow2',
+                'owner_specified.openstack.md5': fakes.NO_MD5,
+                'owner_specified.openstack.sha256': fakes.NO_SHA256,
+                'owner_specified.openstack.object': 'images/{name}'.format(
+                    name=self.image_name),
+                'visibility': 'private'}
+
+        ret = args.copy()
+        ret['id'] = self.image_id
+        ret['status'] = 'success'
+        ret['checksum'] = 'fake'
+
+        self.register_uris([
+            dict(method='GET',
+                 uri='https://image.example.com/v2/images',
+                 json={'images': []}),
+            dict(method='POST',
+                 uri='https://image.example.com/v2/images',
+                 json=ret,
+                 validate=dict(json=args)),
+            dict(method='PUT',
+                 uri='https://image.example.com/v2/images/{id}/file'.format(
+                     id=self.image_id),
+                 status_code=400,
+                 validate=dict(
+                     headers={
+                         'Content-Type': 'application/octet-stream',
+                     },
+                 )),
+            dict(method='DELETE',
+                 uri='https://image.example.com/v2/images/{id}'.format(
+                     id=self.image_id)),
+        ])
+
+        self.assertRaises(
+            exc.OpenStackCloudHTTPError,
+            self._call_create_image,
+            self.image_name,
+            md5='some_fake')
+
+        self.assert_calls()
+
     def test_create_image_put_bad_int(self):
         self.cloud.image_api_use_tasks = False
 
@@ -783,6 +848,11 @@ class TestImage(BaseTestImage):
                          'Content-Type': 'application/octet-stream',
                      },
                  )),
+            dict(method='GET',
+                 uri='https://image.example.com/v2/images/{id}'.format(
+                     id=self.image_id
+                 ),
+                 json=ret),
             dict(method='GET',
                  uri='https://image.example.com/v2/images',
                  json={'images': [ret]}),
@@ -809,6 +879,7 @@ class TestImage(BaseTestImage):
         ret = args.copy()
         ret['id'] = self.image_id
         ret['status'] = 'success'
+        ret['checksum'] = fakes.NO_MD5
 
         self.register_uris([
             dict(method='GET',
@@ -826,6 +897,11 @@ class TestImage(BaseTestImage):
                          'Content-Type': 'application/octet-stream',
                      },
                  )),
+            dict(method='GET',
+                 uri='https://image.example.com/v2/images/{id}'.format(
+                     id=self.image_id
+                 ),
+                 json=ret),
             dict(method='GET',
                  uri='https://image.example.com/v2/images',
                  json={'images': [ret]}),
@@ -853,6 +929,7 @@ class TestImage(BaseTestImage):
         ret = args.copy()
         ret['id'] = self.image_id
         ret['status'] = 'success'
+        ret['checksum'] = fakes.NO_MD5
 
         self.register_uris([
             dict(method='GET',
@@ -870,6 +947,11 @@ class TestImage(BaseTestImage):
                          'Content-Type': 'application/octet-stream',
                      },
                  )),
+            dict(method='GET',
+                 uri='https://image.example.com/v2/images/{id}'.format(
+                     id=self.image_id
+                 ),
+                 json=ret),
             dict(method='GET',
                  uri='https://image.example.com/v2/images',
                  json={'images': [ret]}),
@@ -985,6 +1067,7 @@ class TestImageVolume(BaseTestImage):
     def test_create_image_volume(self):
 
         self.register_uris([
+            self.get_cinder_discovery_mock_dict(),
             dict(method='POST',
                  uri=self.get_mock_url(
                      'volumev2', append=['volumes', self.volume_id, 'action']),
@@ -1016,6 +1099,7 @@ class TestImageVolume(BaseTestImage):
     def test_create_image_volume_duplicate(self):
 
         self.register_uris([
+            self.get_cinder_discovery_mock_dict(),
             dict(method='POST',
                  uri=self.get_mock_url(
                      'volumev2', append=['volumes', self.volume_id, 'action']),
