@@ -644,6 +644,24 @@ class TestResource(base.TestCase):
         self.assertEqual(sot.alt, value2)
         self.assertEqual(sot.id, value2)
 
+    def test__alternate_id_from_other_property(self):
+        class Test(resource.Resource):
+            foo = resource.Body("foo")
+            bar = resource.Body("bar", alternate_id=True)
+
+        # NOTE(redrobot): My expectation looking at the Test class defined
+        # in this test is that because the alternate_id parameter is
+        # is being set to True on the "bar" property of the Test class,
+        # then the _alternate_id() method should return the name of that "bar"
+        # property.
+        self.assertEqual("bar", Test._alternate_id())
+        sot = Test(bar='bunnies')
+        self.assertEqual(sot.id, 'bunnies')
+        self.assertEqual(sot.bar, 'bunnies')
+        sot = Test(id='chickens', bar='bunnies')
+        self.assertEqual(sot.id, 'chickens')
+        self.assertEqual(sot.bar, 'bunnies')
+
     def test__get_id_instance(self):
         class Test(resource.Resource):
             id = resource.Body("id")
@@ -1027,6 +1045,154 @@ class TestResource(base.TestCase):
         sot._body.dirty = mock.Mock(return_value={"x": "y"})
         self.assertRaises(exceptions.MethodNotSupported, sot.commit, "")
 
+    def test_unknown_attrs_under_props_create(self):
+        class Test(resource.Resource):
+            properties = resource.Body("properties")
+            _store_unknown_attrs_as_properties = True
+
+        sot = Test.new(**{
+            'dummy': 'value',
+        })
+        self.assertDictEqual({'dummy': 'value'}, sot.properties)
+        self.assertDictEqual(
+            {'dummy': 'value'}, sot.to_dict()['properties']
+        )
+        self.assertDictEqual(
+            {'dummy': 'value'}, sot['properties']
+        )
+        self.assertEqual('value', sot['properties']['dummy'])
+
+        sot = Test.new(**{
+            'dummy': 'value',
+            'properties': 'a,b,c'
+        })
+        self.assertDictEqual(
+            {'dummy': 'value', 'properties': 'a,b,c'},
+            sot.properties
+        )
+        self.assertDictEqual(
+            {'dummy': 'value', 'properties': 'a,b,c'},
+            sot.to_dict()['properties']
+        )
+
+        sot = Test.new(**{'properties': None})
+        self.assertIsNone(sot.properties)
+        self.assertIsNone(sot.to_dict()['properties'])
+
+    def test_unknown_attrs_not_stored(self):
+        class Test(resource.Resource):
+            properties = resource.Body("properties")
+
+        sot = Test.new(**{
+            'dummy': 'value',
+        })
+        self.assertIsNone(sot.properties)
+
+    def test_unknown_attrs_not_stored1(self):
+        class Test(resource.Resource):
+            _store_unknown_attrs_as_properties = True
+
+        sot = Test.new(**{
+            'dummy': 'value',
+        })
+        self.assertRaises(KeyError, sot.__getitem__, 'properties')
+
+    def test_unknown_attrs_under_props_set(self):
+        class Test(resource.Resource):
+            properties = resource.Body("properties")
+            _store_unknown_attrs_as_properties = True
+
+        sot = Test.new(**{
+            'dummy': 'value',
+        })
+
+        sot['properties'] = {'dummy': 'new_value'}
+        self.assertEqual('new_value', sot['properties']['dummy'])
+        sot.properties = {'dummy': 'new_value1'}
+        self.assertEqual('new_value1', sot['properties']['dummy'])
+
+    def test_unknown_attrs_prepare_request_unpacked(self):
+        class Test(resource.Resource):
+            properties = resource.Body("properties")
+            _store_unknown_attrs_as_properties = True
+
+        # Unknown attribute given as root attribute
+        sot = Test.new(**{
+            'dummy': 'value',
+            'properties': 'a,b,c'
+        })
+
+        request_body = sot._prepare_request(requires_id=False).body
+        self.assertEqual('value', request_body['dummy'])
+        self.assertEqual('a,b,c', request_body['properties'])
+
+        # properties are already a dict
+        sot = Test.new(**{
+            'properties': {
+                'properties': 'a,b,c',
+                'dummy': 'value'
+            }
+        })
+
+        request_body = sot._prepare_request(requires_id=False).body
+        self.assertEqual('value', request_body['dummy'])
+        self.assertEqual('a,b,c', request_body['properties'])
+
+    def test_unknown_attrs_prepare_request_no_unpack_dict(self):
+        # if props type is not None - ensure no unpacking is done
+        class Test(resource.Resource):
+            properties = resource.Body("properties", type=dict)
+        sot = Test.new(**{
+            'properties': {
+                'properties': 'a,b,c',
+                'dummy': 'value'
+            }
+        })
+
+        request_body = sot._prepare_request(requires_id=False).body
+        self.assertDictEqual(
+            {'dummy': 'value', 'properties': 'a,b,c'},
+            request_body['properties'])
+
+    def test_unknown_attrs_prepare_request_patch_unpacked(self):
+        class Test(resource.Resource):
+            properties = resource.Body("properties")
+            _store_unknown_attrs_as_properties = True
+            commit_jsonpatch = True
+
+        sot = Test.existing(**{
+            'dummy': 'value',
+            'properties': 'a,b,c'
+        })
+
+        sot._update(**{'properties': {'dummy': 'new_value'}})
+
+        request_body = sot._prepare_request(requires_id=False, patch=True).body
+        self.assertDictEqual(
+            {
+                u'path': u'/dummy',
+                u'value': u'new_value',
+                u'op': u'replace'
+            },
+            request_body[0])
+
+    def test_unknown_attrs_under_props_translate_response(self):
+        class Test(resource.Resource):
+            properties = resource.Body("properties")
+            _store_unknown_attrs_as_properties = True
+
+        body = {'dummy': 'value', 'properties': 'a,b,c'}
+        response = FakeResponse(body)
+
+        sot = Test()
+
+        sot._translate_response(response, has_body=True)
+
+        self.assertDictEqual(
+            {'dummy': 'value', 'properties': 'a,b,c'},
+            sot.properties
+        )
+
 
 class TestResourceActions(base.TestCase):
 
@@ -1068,6 +1234,8 @@ class TestResourceActions(base.TestCase):
         self.session.post = mock.Mock(return_value=self.response)
         self.session.delete = mock.Mock(return_value=self.response)
         self.session.head = mock.Mock(return_value=self.response)
+        self.session.session = self.session
+        self.session._get_connection = mock.Mock(return_value=self.cloud)
         self.session.default_microversion = None
         self.session.retriable_status_codes = None
 
@@ -1076,16 +1244,18 @@ class TestResourceActions(base.TestCase):
         self.session.get_endpoint_data.return_value = self.endpoint_data
 
     def _test_create(self, cls, requires_id=False, prepend_key=False,
-                     microversion=None):
+                     microversion=None, base_path=None):
         id = "id" if requires_id else None
         sot = cls(id=id)
         sot._prepare_request = mock.Mock(return_value=self.request)
         sot._translate_response = mock.Mock()
 
-        result = sot.create(self.session, prepend_key=prepend_key)
+        result = sot.create(self.session, prepend_key=prepend_key,
+                            base_path=base_path)
 
         sot._prepare_request.assert_called_once_with(
-            requires_id=requires_id, prepend_key=prepend_key)
+            requires_id=requires_id, prepend_key=prepend_key,
+            base_path=base_path)
         if requires_id:
             self.session.put.assert_called_once_with(
                 self.request.url,
@@ -1098,7 +1268,8 @@ class TestResourceActions(base.TestCase):
                 microversion=microversion)
 
         self.assertEqual(sot.microversion, microversion)
-        sot._translate_response.assert_called_once_with(self.response)
+        sot._translate_response.assert_called_once_with(self.response,
+                                                        has_body=sot.has_body)
         self.assertEqual(result, sot)
 
     def test_put_create(self):
@@ -1130,10 +1301,21 @@ class TestResourceActions(base.TestCase):
 
         self._test_create(Test, requires_id=False, prepend_key=True)
 
+    def test_post_create_base_path(self):
+        class Test(resource.Resource):
+            service = self.service_name
+            base_path = self.base_path
+            allow_create = True
+            create_method = 'POST'
+
+        self._test_create(Test, requires_id=False, prepend_key=True,
+                          base_path='dummy')
+
     def test_fetch(self):
         result = self.sot.fetch(self.session)
 
-        self.sot._prepare_request.assert_called_once_with(requires_id=True)
+        self.sot._prepare_request.assert_called_once_with(
+            requires_id=True, base_path=None)
         self.session.get.assert_called_once_with(
             self.request.url, microversion=None)
 
@@ -1154,7 +1336,8 @@ class TestResourceActions(base.TestCase):
 
         result = sot.fetch(self.session)
 
-        sot._prepare_request.assert_called_once_with(requires_id=True)
+        sot._prepare_request.assert_called_once_with(
+            requires_id=True, base_path=None)
         self.session.get.assert_called_once_with(
             self.request.url, microversion='1.42')
 
@@ -1165,7 +1348,20 @@ class TestResourceActions(base.TestCase):
     def test_fetch_not_requires_id(self):
         result = self.sot.fetch(self.session, False)
 
-        self.sot._prepare_request.assert_called_once_with(requires_id=False)
+        self.sot._prepare_request.assert_called_once_with(
+            requires_id=False, base_path=None)
+        self.session.get.assert_called_once_with(
+            self.request.url, microversion=None)
+
+        self.sot._translate_response.assert_called_once_with(self.response)
+        self.assertEqual(result, self.sot)
+
+    def test_fetch_base_path(self):
+        result = self.sot.fetch(self.session, False, base_path='dummy')
+
+        self.sot._prepare_request.assert_called_once_with(
+            requires_id=False,
+            base_path='dummy')
         self.session.get.assert_called_once_with(
             self.request.url, microversion=None)
 
@@ -1175,7 +1371,21 @@ class TestResourceActions(base.TestCase):
     def test_head(self):
         result = self.sot.head(self.session)
 
-        self.sot._prepare_request.assert_called_once_with()
+        self.sot._prepare_request.assert_called_once_with(base_path=None)
+        self.session.head.assert_called_once_with(
+            self.request.url,
+            headers={"Accept": ""},
+            microversion=None)
+
+        self.assertIsNone(self.sot.microversion)
+        self.sot._translate_response.assert_called_once_with(
+            self.response, has_body=False)
+        self.assertEqual(result, self.sot)
+
+    def test_head_base_path(self):
+        result = self.sot.head(self.session, base_path='dummy')
+
+        self.sot._prepare_request.assert_called_once_with(base_path='dummy')
         self.session.head.assert_called_once_with(
             self.request.url,
             headers={"Accept": ""},
@@ -1199,7 +1409,7 @@ class TestResourceActions(base.TestCase):
 
         result = sot.head(self.session)
 
-        sot._prepare_request.assert_called_once_with()
+        sot._prepare_request.assert_called_once_with(base_path=None)
         self.session.head.assert_called_once_with(
             self.request.url,
             headers={"Accept": ""},
@@ -1212,7 +1422,7 @@ class TestResourceActions(base.TestCase):
 
     def _test_commit(self, commit_method='PUT', prepend_key=True,
                      has_body=True, microversion=None,
-                     commit_args=None, expected_args=None):
+                     commit_args=None, expected_args=None, base_path=None):
         self.sot.commit_method = commit_method
 
         # Need to make sot look dirty so we can attempt an update
@@ -1220,10 +1430,11 @@ class TestResourceActions(base.TestCase):
         self.sot._body.dirty = mock.Mock(return_value={"x": "y"})
 
         self.sot.commit(self.session, prepend_key=prepend_key,
-                        has_body=has_body, **(commit_args or {}))
+                        has_body=has_body, base_path=base_path,
+                        **(commit_args or {}))
 
         self.sot._prepare_request.assert_called_once_with(
-            prepend_key=prepend_key)
+            prepend_key=prepend_key, base_path=base_path)
 
         if commit_method == 'PATCH':
             self.session.patch.assert_called_once_with(
@@ -1252,6 +1463,10 @@ class TestResourceActions(base.TestCase):
         self._test_commit(
             commit_method='PATCH', prepend_key=False, has_body=False)
 
+    def test_commit_base_path(self):
+        self._test_commit(commit_method='PUT', prepend_key=True, has_body=True,
+                          base_path='dummy')
+
     def test_commit_patch_retry_on_conflict(self):
         self._test_commit(
             commit_method='PATCH',
@@ -1288,6 +1503,60 @@ class TestResourceActions(base.TestCase):
 
         self.session.put.assert_not_called()
 
+    def test_patch_with_sdk_names(self):
+        class Test(resource.Resource):
+            allow_patch = True
+
+            id = resource.Body('id')
+            attr = resource.Body('attr')
+            nested = resource.Body('renamed')
+            other = resource.Body('other')
+
+        test_patch = [{'path': '/attr', 'op': 'replace', 'value': 'new'},
+                      {'path': '/nested/dog', 'op': 'remove'},
+                      {'path': '/nested/cat', 'op': 'add', 'value': 'meow'}]
+        expected = [{'path': '/attr', 'op': 'replace', 'value': 'new'},
+                    {'path': '/renamed/dog', 'op': 'remove'},
+                    {'path': '/renamed/cat', 'op': 'add', 'value': 'meow'}]
+        sot = Test.existing(id=1, attr=42, nested={'dog': 'bark'})
+        sot.patch(self.session, test_patch)
+        self.session.patch.assert_called_once_with(
+            '/1', json=expected, headers=mock.ANY, microversion=None)
+
+    def test_patch_with_server_names(self):
+        class Test(resource.Resource):
+            allow_patch = True
+
+            id = resource.Body('id')
+            attr = resource.Body('attr')
+            nested = resource.Body('renamed')
+            other = resource.Body('other')
+
+        test_patch = [{'path': '/attr', 'op': 'replace', 'value': 'new'},
+                      {'path': '/renamed/dog', 'op': 'remove'},
+                      {'path': '/renamed/cat', 'op': 'add', 'value': 'meow'}]
+        sot = Test.existing(id=1, attr=42, nested={'dog': 'bark'})
+        sot.patch(self.session, test_patch)
+        self.session.patch.assert_called_once_with(
+            '/1', json=test_patch, headers=mock.ANY, microversion=None)
+
+    def test_patch_with_changed_fields(self):
+        class Test(resource.Resource):
+            allow_patch = True
+
+            attr = resource.Body('attr')
+            nested = resource.Body('renamed')
+            other = resource.Body('other')
+
+        sot = Test.existing(id=1, attr=42, nested={'dog': 'bark'})
+        sot.attr = 'new'
+        sot.patch(self.session, {'path': '/renamed/dog', 'op': 'remove'})
+
+        expected = [{'path': '/attr', 'op': 'replace', 'value': 'new'},
+                    {'path': '/renamed/dog', 'op': 'remove'}]
+        self.session.patch.assert_called_once_with(
+            '/1', json=expected, headers=mock.ANY, microversion=None)
+
     def test_delete(self):
         result = self.sot.delete(self.session)
 
@@ -1393,6 +1662,7 @@ class TestResourceActions(base.TestCase):
         mock_response = mock.Mock()
         mock_response.status_code = 200
         mock_response.json.return_value = {key: [{"id": id_value}]}
+        mock_response.links = []
 
         self.session.get.return_value = mock_response
 
@@ -1485,6 +1755,53 @@ class TestResourceActions(base.TestCase):
         self.assertEqual(2, len(self.session.get.call_args_list))
         self.assertIsInstance(results[0], self.test_class)
 
+    def test_list_response_paginated_with_links_and_query(self):
+        q_limit = 1
+        ids = [1, 2]
+        mock_response = mock.Mock()
+        mock_response.status_code = 200
+        mock_response.links = {}
+        mock_response.json.side_effect = [
+            {
+                "resources": [{"id": ids[0]}],
+                "resources_links": [{
+                    "href": "https://example.com/next-url?limit=%d" % q_limit,
+                    "rel": "next",
+                }]
+            }, {
+                "resources": [{"id": ids[1]}],
+            }, {
+                "resources": [],
+            }]
+
+        self.session.get.return_value = mock_response
+
+        class Test(self.test_class):
+            _query_mapping = resource.QueryParameters("limit")
+
+        results = list(Test.list(self.session, paginated=True, limit=q_limit))
+
+        self.assertEqual(2, len(results))
+        self.assertEqual(ids[0], results[0].id)
+        self.assertEqual(ids[1], results[1].id)
+        self.assertEqual(
+            mock.call('base_path',
+                      headers={'Accept': 'application/json'}, params={
+                          'limit': q_limit,
+                      },
+                      microversion=None),
+            self.session.get.mock_calls[0])
+        self.assertEqual(
+            mock.call('https://example.com/next-url',
+                      headers={'Accept': 'application/json'}, params={
+                          'limit': [str(q_limit)],
+                      },
+                      microversion=None),
+            self.session.get.mock_calls[2])
+
+        self.assertEqual(3, len(self.session.get.call_args_list))
+        self.assertIsInstance(results[0], self.test_class)
+
     def test_list_response_paginated_with_microversions(self):
         class Test(resource.Resource):
             service = self.service_name
@@ -1610,6 +1927,33 @@ class TestResourceActions(base.TestCase):
         except exceptions.InvalidResourceQuery as err:
             self.assertEqual(str(err), 'Invalid query params: something_wrong')
 
+    def test_allow_invalid_list_params(self):
+        qp = "query param!"
+        qp_name = "query-param"
+        uri_param = "uri param!"
+
+        mock_empty = mock.Mock()
+        mock_empty.status_code = 200
+        mock_empty.links = {}
+        mock_empty.json.return_value = {"resources": []}
+
+        self.session.get.side_effect = [mock_empty]
+
+        class Test(self.test_class):
+            _query_mapping = resource.QueryParameters(query_param=qp_name)
+            base_path = "/%(something)s/blah"
+            something = resource.URI("something")
+
+        list(Test.list(self.session, paginated=True, query_param=qp,
+                       allow_unknown_params=True, something=uri_param,
+                       something_wrong=True))
+        self.session.get.assert_called_once_with(
+            "/{something}/blah".format(something=uri_param),
+            headers={'Accept': 'application/json'},
+            microversion=None,
+            params={qp_name: qp}
+        )
+
     def test_values_as_list_params(self):
         id = 1
         qp = "query param!"
@@ -1852,7 +2196,10 @@ class TestResourceActions(base.TestCase):
             microversion=None)
 
         # Ensure we're done after those three items
-        self.assertRaises(StopIteration, next, results)
+        # In python3.7, PEP 479 is enabled for all code, and StopIteration
+        # raised directly from code is turned into a RuntimeError.
+        # Something about how mock is implemented triggers that here.
+        self.assertRaises((StopIteration, RuntimeError), next, results)
 
         # Ensure we only made two calls to get this done
         self.assertEqual(3, len(self.session.get.call_args_list))
@@ -1958,38 +2305,42 @@ class TestResourceActions(base.TestCase):
 
 class TestResourceFind(base.TestCase):
 
+    result = 1
+
+    class Base(resource.Resource):
+
+        @classmethod
+        def existing(cls, **kwargs):
+            response = mock.Mock()
+            response.status_code = 404
+            raise exceptions.ResourceNotFound(
+                'Not Found', response=response)
+
+        @classmethod
+        def list(cls, session, **params):
+            return None
+
+    class OneResult(Base):
+
+        @classmethod
+        def _get_one_match(cls, *args):
+            return TestResourceFind.result
+
+    class NoResults(Base):
+
+        @classmethod
+        def _get_one_match(cls, *args):
+            return None
+
+    class OneResultWithQueryParams(OneResult):
+
+        _query_mapping = resource.QueryParameters('name')
+
     def setUp(self):
         super(TestResourceFind, self).setUp()
-
-        self.result = 1
-
-        class Base(resource.Resource):
-
-            @classmethod
-            def existing(cls, **kwargs):
-                response = mock.Mock()
-                response.status_code = 404
-                raise exceptions.ResourceNotFound(
-                    'Not Found', response=response)
-
-            @classmethod
-            def list(cls, session):
-                return None
-
-        class OneResult(Base):
-
-            @classmethod
-            def _get_one_match(cls, *args):
-                return self.result
-
-        class NoResults(Base):
-
-            @classmethod
-            def _get_one_match(cls, *args):
-                return None
-
-        self.no_results = NoResults
-        self.one_result = OneResult
+        self.no_results = self.NoResults
+        self.one_result = self.OneResult
+        self.one_result_with_qparams = self.OneResultWithQueryParams
 
     def test_find_short_circuit(self):
         value = 1
@@ -2002,20 +2353,37 @@ class TestResourceFind(base.TestCase):
                 mock_match.fetch.return_value = value
                 return mock_match
 
-        result = Test.find("session", "name")
+        result = Test.find(self.cloud.compute, "name")
 
         self.assertEqual(result, value)
 
     def test_no_match_raise(self):
         self.assertRaises(exceptions.ResourceNotFound, self.no_results.find,
-                          "session", "name", ignore_missing=False)
+                          self.cloud.compute, "name", ignore_missing=False)
 
     def test_no_match_return(self):
         self.assertIsNone(
-            self.no_results.find("session", "name", ignore_missing=True))
+            self.no_results.find(
+                self.cloud.compute, "name", ignore_missing=True))
 
-    def test_find_result(self):
-        self.assertEqual(self.result, self.one_result.find("session", "name"))
+    def test_find_result_name_not_in_query_parameters(self):
+        with mock.patch.object(self.one_result, 'existing',
+                               side_effect=self.OneResult.existing) \
+                as mock_existing, \
+                mock.patch.object(self.one_result, 'list',
+                                  side_effect=self.OneResult.list) \
+                as mock_list:
+            self.assertEqual(
+                self.result,
+                self.one_result.find(self.cloud.compute, "name"))
+            mock_existing.assert_called_once_with(id='name',
+                                                  connection=mock.ANY)
+            mock_list.assert_called_once_with(mock.ANY)
+
+    def test_find_result_name_in_query_parameters(self):
+        self.assertEqual(
+            self.result,
+            self.one_result_with_qparams.find(self.cloud.compute, "name"))
 
     def test_match_empty_results(self):
         self.assertIsNone(resource.Resource._get_one_match("name", []))
@@ -2080,7 +2448,7 @@ class TestWaitForStatus(base.TestCase):
         res.status = status
 
         result = resource.wait_for_status(
-            "session", res, status, "failures", "interval", "wait")
+            self.cloud.compute, res, status, "failures", "interval", "wait")
 
         self.assertTrue(result, res)
 
@@ -2090,7 +2458,7 @@ class TestWaitForStatus(base.TestCase):
         res.status = status
 
         result = resource.wait_for_status(
-            "session", res, 'lOling', "failures", "interval", "wait")
+            self.cloud.compute, res, 'lOling', "failures", "interval", "wait")
 
         self.assertTrue(result, res)
 
@@ -2100,7 +2468,7 @@ class TestWaitForStatus(base.TestCase):
         res.mood = status
 
         result = resource.wait_for_status(
-            "session", res, status, "failures", "interval", "wait",
+            self.cloud.compute, res, status, "failures", "interval", "wait",
             attribute='mood')
 
         self.assertTrue(result, res)
@@ -2190,7 +2558,7 @@ class TestWaitForStatus(base.TestCase):
 
         self.assertRaises(exceptions.ResourceTimeout,
                           resource.wait_for_status,
-                          "session", res, status, None, 0.01, 0.1)
+                          self.cloud.compute, res, status, None, 0.01, 0.1)
 
     def test_no_sleep(self):
         res = mock.Mock()
@@ -2199,7 +2567,7 @@ class TestWaitForStatus(base.TestCase):
 
         self.assertRaises(exceptions.ResourceTimeout,
                           resource.wait_for_status,
-                          "session", res, "status", None, 0, -1)
+                          self.cloud.compute, res, "status", None, 0, -1)
 
 
 class TestWaitForDelete(base.TestCase):
@@ -2213,7 +2581,7 @@ class TestWaitForDelete(base.TestCase):
             None, None,
             exceptions.ResourceNotFound('Not Found', response)]
 
-        result = resource.wait_for_delete("session", res, 1, 3)
+        result = resource.wait_for_delete(self.cloud.compute, res, 1, 3)
 
         self.assertEqual(result, res)
 
@@ -2225,7 +2593,7 @@ class TestWaitForDelete(base.TestCase):
         self.assertRaises(
             exceptions.ResourceTimeout,
             resource.wait_for_delete,
-            "session", res, 0.1, 0.3)
+            self.cloud.compute, res, 0.1, 0.3)
 
 
 @mock.patch.object(resource.Resource, '_get_microversion_for', autospec=True)
